@@ -2,17 +2,18 @@
 #'
 #' This function uses a while loop to continually collect children taxa down
 #' to the taxonomic rank that you specify in the `downto` parameter. You
-#' can get data from ITIS (itis), Catalogue of Life (col), GBIF (gbif),
-#' NCBI (ncbi) or WORMS (worms). There is no method exposed by these four 
-#' services for getting taxa at a specific taxonomic rank, so we do it 
+#' can get data from ITIS (itis), GBIF (gbif), NCBI (ncbi), WORMS (worms),
+#' or BOLD (bold). There is no method exposed by these four
+#' services for getting taxa at a specific taxonomic rank, so we do it
 #' ourselves here.
 #'
 #' @export
-#' @param x Vector of taxa names (character) or IDs (character or numeric)
+#' @param sci_id Vector of taxa names (character) or IDs (character or numeric)
 #' to query.
-#' @param db character; database to query. One or more of `itis`, `col`, `gbif`,
-#' `ncbi` or `worms`. Note that each taxonomic  data source has their own
-#' identifiers, so that if you provide the wrong `db` value for the identifier
+#' @param db character; database to query. One or more of `itis`, `gbif`,
+#' `ncbi`, `worms`, or `bold`. Note that each taxonomic  data source has
+#' their own identifiers, so that if you provide the wrong `db` value for
+#' the identifier
 #' you could get a result, but it will likely be wrong (not what you were
 #' expecting). If using ncbi, we recommend getting an API key; see
 #' [taxize-authentication]
@@ -28,60 +29,49 @@
 #' intermediate taxonomic groups. Default: `FALSE`
 #' @param rows (numeric) Any number from 1 to infinity. If the default NA, all
 #' rows are considered. Note that this parameter is ignored if you pass in a
-#' taxonomic id of any of the acceptable classes: tsn, colid.
+#' taxonomic id of any of the acceptable classes: tsn.
 #' @param limit Number of records to return
 #' @param start Record number to start at
+#' @param x Deprecated, see `sci_id`
 #' @param ... Further args passed on to [itis_downstream()],
-#' [col_downstream()], [gbif_downstream()], [ncbi_downstream()],
-#' or [worms_downstream()]
+#' [gbif_downstream()], [ncbi_downstream()],
+#' [worms_downstream()], or [bold_downstream()]
 #'
 #' @return A named list of data.frames with the downstream names of every
 #' supplied taxa. You get an NA if there was no match in the database.
-#' 
+#'
 #' @section Authentication:
 #' See [taxize-authentication] for help on authentication
+#' 
+#' @section bold:
+#' BEWARE: `db="bold"` scrapes the BOLD website, so may be unstable. That is,
+#' one day it may work, and the next it may fail. Open an issue if you
+#' encounter an error: https://github.com/ropensci/taxize/issues
 #'
 #' @examples \dontrun{
 #' # Plug in taxon IDs
-#' downstream("015be25f6b061ba517f495394b80f108", db = "col",
-#'   downto = "species")
 #' downstream(125732, db = 'worms', downto = 'species')
+#' downstream(3451, db = 'bold', downto = 'species')
 #'
 #' # Plug in taxon names
-#' downstream("Insecta", db = 'col', downto = 'order')
-#' downstream("Apis", db = 'col', downto = 'species')
 #' downstream("Apis", db = 'ncbi', downto = 'species')
 #' downstream("Apis", db = 'itis', downto = 'species')
+#' downstream("Apis", db = 'bold', downto = 'species')
 #' downstream("Gadus", db = 'worms', downto = 'species')
 #' downstream(c("Apis","Epeoloides"), db = 'itis', downto = 'species')
-#' downstream(c("Apis","Epeoloides"), db = 'col', downto = 'species')
 #' downstream("Ursus", db = 'gbif', downto = 'species')
 #' downstream(get_gbifid("Ursus"), db = 'gbif', downto = 'species')
 #'
-#' # Plug in IDs
-#' id <- get_colid("Apis")
-#' downstream(id, downto = 'species')
-#'
-#' ## Equivalently, plug in the call to get the id via e.g., get_colid
-#' ## into downstream
-#' identical(downstream(id, downto = 'species'),
-#'          downstream(get_colid("Apis"), downto = 'species'))
-#'
-#' id <- get_colid("Apis")
-#' downstream(id, downto = 'species')
-#' downstream(get_colid("Apis"), downto = 'species')
-#'
 #' # Many taxa
 #' sp <- names_list("genus", 3)
-#' downstream(sp, db = 'col', downto = 'species')
 #' downstream(sp, db = 'itis', downto = 'species')
 #' downstream(sp, db = 'gbif', downto = 'species')
 #'
 #' # Both data sources
-#' ids <- get_ids("Apis", db = c('col','itis'))
+#' ids <- get_ids("Apis", db = c('gbif','itis'))
 #' downstream(ids, downto = 'species')
 #' ## same result
-#' downstream(get_ids("Apis", db = c('col','itis')), downto = 'species')
+#' downstream(get_ids("Apis", db = c('gbif','itis')), downto = 'species')
 #'
 #' # Collect intermediate names
 #' ## itis
@@ -89,17 +79,14 @@
 #' downstream('Bangiophyceae', db="itis", downto="genus", intermediate=TRUE)
 #' downstream(get_tsn('Bangiophyceae'), downto="genus")
 #' downstream(get_tsn('Bangiophyceae'), downto="genus", intermediate=TRUE)
-#' ## col
-#' downstream(get_colid("Animalia"), downto="class")
-#' downstream(get_colid("Animalia"), downto="class", intermediate=TRUE)
 #'
 #' # Use the rows parameter
 #' ## note how in the second function call you don't get the prompt
-#' downstream("Poa", db = 'col', downto="species")
-#' downstream("Poa", db = 'col', downto="species", rows=1)
+#' downstream("Poa", db = 'gbif', downto="species")
+#' downstream("Poa", db = 'gbif', downto="species", rows=1)
 #'
 #' # use curl options
-#' res <- downstream("Apis", db = 'col', downto = 'species', verbose = TRUE)
+#' res <- downstream("Apis", db = 'gbif', downto = 'species', verbose = TRUE)
 #' }
 downstream <- function(...){
   UseMethod("downstream")
@@ -107,36 +94,38 @@ downstream <- function(...){
 
 #' @export
 #' @rdname downstream
-downstream.default <- function(x, db = NULL, downto = NULL,
-                               intermediate = FALSE, rows=NA, ...) {
+downstream.default <- function(sci_id, db=NULL, downto=NULL,
+                               intermediate=FALSE, rows=NA, x=NULL, ...) {
   nstop(downto, "downto")
   nstop(db)
+  pchk(x, "sci_id")
+  if (!is.null(x)) sci_id <- x
   switch(
     db,
     itis = {
-      id <- process_stream_ids(x, db, get_tsn, rows = rows, ...)
+      id <- process_stream_ids(sci_id, db, get_tsn, rows = rows, ...)
       stats::setNames(downstream(id, downto = tolower(downto),
-                                 intermediate = intermediate, ...), x)
-    },
-    col = {
-      id <- process_stream_ids(x, db, get_colid, rows = rows, ...)
-      stats::setNames(downstream(id, downto = tolower(downto),
-                                 intermediate = intermediate, ...), x)
+                                 intermediate = intermediate, ...), sci_id)
     },
     gbif = {
-      id <- process_stream_ids(x, db, get_gbifid, rows = rows, ...)
+      id <- process_stream_ids(sci_id, db, get_gbifid, rows = rows, ...)
       stats::setNames(downstream(id, downto = tolower(downto),
-                                 intermediate = intermediate, ...), x)
+                                 intermediate = intermediate, ...), sci_id)
     },
     ncbi = {
-      id <- process_stream_ids(x, db, get_uid, rows = rows, ...)
+      id <- process_stream_ids(sci_id, db, get_uid, rows = rows, ...)
       stats::setNames(downstream(id, downto = tolower(downto),
-                                 intermediate = intermediate, ...), x)
+                                 intermediate = intermediate, ...), sci_id)
     },
     worms = {
-      id <- process_stream_ids(x, db, get_wormsid, rows = rows, ...)
+      id <- process_stream_ids(sci_id, db, get_wormsid, rows = rows, ...)
       stats::setNames(downstream(id, downto = tolower(downto),
-                                 intermediate = intermediate, ...), x)
+                                 intermediate = intermediate, ...), sci_id)
+    },
+    bold = {
+      id <- process_stream_ids(as.character(sci_id), db, get_boldid, rows = rows, ...)
+      stats::setNames(downstream(id, downto = tolower(downto),
+                                 intermediate = intermediate, ...), sci_id)
     },
     stop("the provided db value was not recognised/is not supported",
          call. = FALSE)
@@ -150,8 +139,8 @@ process_stream_ids <- function(input, db, fxn, ...){
     is.character(input) &&
     all(grepl("[[:digit:]]", input))
   ) {
-    as_fxn <- switch(db, itis = as.tsn, col = as.colid, gbif = as.gbifid,
-      ncbi = as.uid, worms = as.wormsid)
+    as_fxn <- switch(db, itis = as.tsn, gbif = as.gbifid,
+      ncbi = as.uid, worms = as.wormsid, bold = as.boldid)
     as_fxn(input, check = FALSE)
   } else {
     eval(fxn)(input, ...)
@@ -160,7 +149,7 @@ process_stream_ids <- function(input, db, fxn, ...){
 
 #' @export
 #' @rdname downstream
-downstream.tsn <- function(x, db = NULL, downto = NULL,
+downstream.tsn <- function(sci_id, db = NULL, downto = NULL,
                            intermediate = FALSE, ...) {
   warn_db(list(db = db), "itis")
   fun <- function(y, downto, intermediate, ...) {
@@ -168,34 +157,17 @@ downstream.tsn <- function(x, db = NULL, downto = NULL,
     if (is.na(y)) {
       NA
     } else {
-		  itis_downstream(tsns = y, downto = downto,
+		  itis_downstream(y, downto = downto,
 		                  intermediate = intermediate, ...)
     }
   }
-  out <- lapply(x, fun, downto = downto, intermediate = intermediate, ...)
-  structure(out, class = 'downstream', db = 'itis', .Names = x)
+  out <- lapply(sci_id, fun, downto = downto, intermediate = intermediate, ...)
+  structure(out, class = 'downstream', db = 'itis', .Names = sci_id)
 }
 
 #' @export
 #' @rdname downstream
-downstream.colid <- function(x, db = NULL, downto = NULL,
-                             intermediate = FALSE, ...) {
-  warn_db(list(db = db), "col")
-  fun <- function(y, downto, intermediate, ...){
-    # return NA if NA is supplied
-    if (is.na(y)) {
-      NA
-    } else {
-      col_downstream(id = y, downto = downto, intermediate = intermediate, ...)
-    }
-  }
-  out <- lapply(x, fun, downto = downto, intermediate = intermediate, ...)
-  structure(simp(out), class = 'downstream', db = 'col')
-}
-
-#' @export
-#' @rdname downstream
-downstream.gbifid <- function(x, db = NULL, downto = NULL,
+downstream.gbifid <- function(sci_id, db = NULL, downto = NULL,
                               intermediate = FALSE, limit = 100,
                               start = NULL, ...) {
   warn_db(list(db = db), "gbif")
@@ -204,19 +176,19 @@ downstream.gbifid <- function(x, db = NULL, downto = NULL,
     if (is.na(y)) {
       NA
     } else {
-      gbif_downstream(key = y, downto = downto,
+      gbif_downstream(y, downto = downto,
                       intermediate = intermediate, limit = limit,
                       start = start, ...)
     }
   }
-  out <- lapply(x, fun, downto = downto, intermediate = intermediate,
+  out <- lapply(sci_id, fun, downto = downto, intermediate = intermediate,
     limit = limit, start = start, ...)
   structure(out, class = 'downstream', db = 'gbif')
 }
 
 #' @export
 #' @rdname downstream
-downstream.uid <- function(x, db = NULL, downto = NULL,
+downstream.uid <- function(sci_id, db = NULL, downto = NULL,
                               intermediate = FALSE, ...) {
   warn_db(list(db = db), "ncbi")
   fun <- function(y, downto, intermediate, ...){
@@ -228,13 +200,13 @@ downstream.uid <- function(x, db = NULL, downto = NULL,
                       intermediate = intermediate, ...)
     }
   }
-  out <- lapply(x, fun, downto = downto, intermediate = intermediate, ...)
+  out <- lapply(sci_id, fun, downto = downto, intermediate = intermediate, ...)
   structure(out, class = 'downstream', db = 'ncbi')
 }
 
 #' @export
 #' @rdname downstream
-downstream.wormsid <- function(x, db = NULL, downto = NULL,
+downstream.wormsid <- function(sci_id, db = NULL, downto = NULL,
                               intermediate = FALSE, ...) {
   warn_db(list(db = db), "worms")
   fun <- function(y, downto, intermediate, ...){
@@ -246,13 +218,31 @@ downstream.wormsid <- function(x, db = NULL, downto = NULL,
                       intermediate = intermediate, ...)
     }
   }
-  out <- lapply(x, fun, downto = downto, intermediate = intermediate, ...)
+  out <- lapply(sci_id, fun, downto = downto, intermediate = intermediate, ...)
   structure(out, class = 'downstream', db = 'worms')
 }
 
 #' @export
 #' @rdname downstream
-downstream.ids <- function(x, db = NULL, downto = NULL,
+downstream.boldid <- function(sci_id, db = NULL, downto = NULL,
+                              intermediate = FALSE, ...) {
+  warn_db(list(db = db), "bold")
+  fun <- function(y, downto, intermediate, ...){
+    # return NA if NA is supplied
+    if (is.na(y)) {
+      NA
+    } else {
+      bold_downstream(id = y, downto = downto,
+                      intermediate = intermediate, ...)
+    }
+  }
+  out <- lapply(sci_id, fun, downto = downto, intermediate = intermediate, ...)
+  structure(out, class = 'downstream', db = 'bold')
+}
+
+#' @export
+#' @rdname downstream
+downstream.ids <- function(sci_id, db = NULL, downto = NULL,
                            intermediate = FALSE, ...) {
   fun <- function(y, downto, intermediate, ...){
     # return NA if NA is supplied
@@ -262,7 +252,7 @@ downstream.ids <- function(x, db = NULL, downto = NULL,
       downstream(y, downto = downto, intermediate = intermediate, ...)
     }
   }
-  structure(lapply(x, fun, downto = downto, intermediate = intermediate, ...),
+  structure(lapply(sci_id, fun, downto = downto, intermediate = intermediate, ...),
             class = 'downstream_ids')
 }
 
